library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_misc.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
USE work.reg_pack.ALL;

library unisim;
use unisim.vcomponents.all;
library work;

entity uart_top_ro is
   port (
         brd_clk : in std_logic;
         rxfifo_dat    : OUT STD_LOGIC_VECTOR (7 downto 0);
         rx : in std_logic;
         tx : out std_logic;
         dat_received  : OUT std_logic;
         debug  : OUT STD_LOGIC_VECTOR (15 downto 0)
         );
end entity;

architecture arch_uart_top_ro of uart_top_ro is

-- Pragma Added to supress synth warnings
--attribute DowngradeIPIdentifiedWarnings: string;
--attribute DowngradeIPIdentifiedWarnings of impl : architecture is "yes";


component axi_uartlite_0 is
  PORT (
-- System signals
      s_axi_aclk           : in  std_logic;
      s_axi_aresetn         : in  std_logic;
      interrupt             : out std_logic;
-- AXI signals
      s_axi_awaddr          : in  std_logic_vector(3 downto 0);
      s_axi_awvalid         : in  std_logic;
      s_axi_awready         : out std_logic;
      s_axi_wdata           : in  std_logic_vector
                              (31 downto 0);
      s_axi_wstrb           : in  std_logic_vector
                              (3 downto 0);
      s_axi_wvalid          : in  std_logic;
      s_axi_wready          : out std_logic;
      s_axi_bresp           : out std_logic_vector(1 downto 0);
      s_axi_bvalid          : out std_logic;
      s_axi_bready          : in  std_logic;
      s_axi_araddr          : in  std_logic_vector(3 downto 0);
      s_axi_arvalid         : in  std_logic;
      s_axi_arready         : out std_logic;
      s_axi_rdata           : out std_logic_vector
                              (31 downto 0);
      s_axi_rresp           : out std_logic_vector(1 downto 0);
      s_axi_rvalid          : out std_logic;
      s_axi_rready          : in  std_logic;

-- UARTLite Interface Signals
      rx                    : in  std_logic;
      tx                    : out std_logic
   );
end component;

---------------------------------------------------------------
--Component Declaration reset_gen_ctr
component reset_gen_ctr is
port (
      ck    : in  STD_LOGIC;
      axi_nres : OUT  STD_LOGIC 
      );
end component;


signal    m_axi_lite_awready          :  std_logic                         ;-- AXI4-Lite
signal    m_axi_lite_awvalid          :  std_logic                         ;-- AXI4-Lite
signal    m_axi_lite_awaddr           :  std_logic_vector (3 downto 0);-- AXI4-Lite
signal    m_axi_lite_wready           :  std_logic                         ;-- AXI4-Lite
signal    m_axi_lite_wvalid           :  std_logic                         ;-- AXI4-Lite
signal    m_axi_lite_wdata            :  std_logic_vector (31 downto 0);-- AXI4-Lite
signal    m_axi_lite_bready           :  std_logic                         ;-- AXI4-Lite
signal    m_axi_lite_bvalid           :  std_logic                         ;-- AXI4-Lite
signal    m_axi_lite_bresp            :  std_logic_vector(1 downto 0)      ;-- AXI4-Lite
signal    s_axi_lite_arready          :  std_logic                         ;-- AXI4-Lite
signal    s_axi_lite_arvalid          :  std_logic                         ;-- AXI4-Lite
signal    s_axi_lite_araddr           :  std_logic_vector (3 downto 0);-- AXI4-Lite
signal    s_axi_lite_rready           :  std_logic                         ;-- AXI4-Lite
signal    s_axi_lite_rvalid           :  std_logic                         ;-- AXI4-Lite
signal    s_axi_lite_rdata            :  std_logic_vector (31 downto 0);-- AXI4-Lite
signal    s_axi_lite_rresp            :  std_logic_vector(1 downto 0)      ;-- AXI4-Lite

signal clk:  std_logic;
signal reset : std_logic;

---
begin
clkf_buf: BUFG
    port map (
      I => brd_clk,
      O => clk
    );
    
i_res_gen_ctr: reset_gen_ctr 
    port map (
          ck        => clk,
          axi_nres  => reset
          );                    
ual : axi_uartlite_0
    PORT MAP (
    interrupt     => open,       
    s_axi_aclk       => clk,
    s_axi_aresetn    => reset,
    s_axi_awaddr     => m_axi_lite_awaddr (3 downto 0),
    s_axi_awvalid    => m_axi_lite_awvalid,
    s_axi_awready    => m_axi_lite_awready,
    s_axi_wdata      => m_axi_lite_wdata,
    s_axi_wstrb      => "1111",
    s_axi_wvalid     => m_axi_lite_wvalid,
    s_axi_wready     => m_axi_lite_wready,
    s_axi_bresp      => m_axi_lite_bresp,
    s_axi_bvalid     => m_axi_lite_bvalid,
    s_axi_bready     => m_axi_lite_bready,
    s_axi_araddr     => s_axi_lite_araddr (3 downto 0),
    s_axi_arvalid    => s_axi_lite_arvalid,
    s_axi_arready    => s_axi_lite_arready,
    s_axi_rdata      => s_axi_lite_rdata,
    s_axi_rresp      => s_axi_lite_rresp,
    s_axi_rvalid     => s_axi_lite_rvalid,
    s_axi_rready     => s_axi_lite_rready,
    rx               => rx,    -- these will go to board 
    tx               => tx_int
    );    
    tx <= tx_int;
---------------------------------------------------------------
--Erst mal alles ruhigstellen:
      m_axi_lite_awaddr (3 downto 0) <= "0000";
      m_axi_lite_awvalid <= '0';
      m_axi_lite_wdata   <= (others => '0');
      m_axi_lite_wvalid  <= '0';
      m_axi_lite_wready  <= '0';
      m_axi_lite_bready  <= '0';


--debug, an Pmod-Steckern angeschlossen:
debug(0) <= rx;  
debug(1) <= '0';
debug(2) <= s_axi_lite_arvalid;
debug(3) <= s_axi_lite_arready;
debug(4) <= s_axi_lite_araddr (3);
debug(5) <= s_axi_lite_araddr (1);
debug(6) <= s_axi_lite_araddr (0);
debug(7) <= dat_received_int;
debug(15 downto 8) <=  rxfifo_dat_int;

end arch_uart_top_ro;

